/**************/
/* show_man.c */
/****************************************************************************/
/*
** Malliohjelma "animaatiosta":
** Tikku-ukko osoittaa oikealla kdelln miss hiiri menee.
**
** Kytt: Tee projekti show_man.prj jossa on:
**           samplew.c
**           show_man.c
**           samplew.def
**         ja aja.
**
**
** Vesa Lappalainen 15.8.1992
**
** Animaation idea:
**   1. kuva on piirretty valmiiksi
**   2. kun jotakin osaa kuvasta pit siirt, pyyhitn vanha osa pois
**   3. lasketaan osan uusi paikka ja/tai muoto
**   4. piirretn uudistettu osa
**
**   "Rautalankamallissa" pyyhkiminen ky siten, ett piirretn
**   kuvan osa kynll, joka muuttaa kaikki pisteet pinvastaisen
**   vrisiksi (mm. musta piste muuttuu valkoiseksi ja pinvastoin).
**   Kun tllaisella kynll piirretn kaksi kertaa sama kuva, ei
**   tuloksena olekaan mitn, siis joka toisella kerralla kuva hvi.
**
**   Monimutkaisemmassa tapauksessa kuvan osan alla oleva taustakuva
**   pit ensin tallettaa ja sitten piirt vasta kuvan osa.
**   Siirrossa tausta palautetaan ja luetaan uusi tausta uudesta
**   paikasta ja sitten piirretn uusi osa.
**
** Tehtvi: 1)  Ukon ksi saadaan monistumaan seuraavasti:
**                 - siir ja muotoile ikkuna siten, ett se menee aavistuksen
**                   toisen ikkunan plle (esim. Program Managerin)
**                 - aktivoi tm toinen ikkuna hiirell
**                 - siirr hiiri ylkautta ukko-ikkunan otsikkokenttn
**                 - paina hiirt aktivoidaksesi ukko-ikkunan
**                 - siirry ukko-ikkunaan ja nyt ukolla pitisi olla 3 ktt
**               Miksi?  Miten vika voitaisiin korjata?
**
**           2)  Muuta ohjelmaa siten, ett kun hiiren nappi painetaan
**               alas, kntyy vasen ksi osoittamaan hiiren suuntaa
**               painamishetkell.  Oikea ksi osoittaa koko ajan.
**
**           3)  Tee sama vasemmalle jalalle.  Miten muuttujien ja tyyppien
**               nimet tllin kannattaa muuttaa?
**
**           4)  Muuta ohjelmaa siten, ett aluksi oikea ksi osoittaa kursoria
**               Kun nappi painastaan alas, lukittuu oikea ksi paikalleen ja
**               oikea jalka rupeaa osoittamaan hiirt niin kauan kun nappi
**               on alhaalla.  Kun nappi pstet yls, alkaa vasen ksi
**               osoittamaan kursoria ja kun nappi painetaan alas alkaa
**               vasen jalka ja kun nappi pstetn yls, ollaan jlleen
**               alkuperisess tilanteessa, eli oikea ksi osoittaa jne...
**
**
----------------------------------------------------------------------------*/
#include <windows.h>
#include <math.h>

char *WindowName = "Nyttj-ukko";

typedef struct {
  int    pituus;
  double suunta;
  POINT  alku;
} kasi_tyyppi;


static kasi_tyyppi Oikea = { 80, 0.0, {60,90} };

/****************************************************************************/
#define POINT_TO_2(pt) pt.x,pt.y  /* Pisteest 2 erillist lukua            */

#define POINT_PLUS_RA(pt,r,a) \
           (int) (pt.x+r*cos(a)), (int) (pt.y-r*sin(a)) /* Piste + r*suunta */


/****************************************************************************/
void PiirraNotKasi(HDC hdc,kasi_tyyppi *kasi)
/* Piirretn ukon oikea ksi NOT-kynll.                                  */
{
/*  SelectObject(hdc,GetStockObject(NULL_BRUSH)); */
  SetROP2(hdc,R2_NOT);
  MoveTo(hdc,POINT_TO_2(kasi->alku));
  LineTo(hdc,POINT_PLUS_RA(kasi->alku,kasi->pituus,kasi->suunta));

}


/****************************************************************************/
int muuta_kaden_suunta(HDC hDC, LONG lParam, kasi_tyyppi *kasi)
{
  int dx,dy;
  POINT pt = MAKEPOINT(lParam);

  PiirraNotKasi(hDC,kasi);   /* Pyyhitn vanha ksi  pois    */

  dx = pt.x - kasi->alku.x;
  dy = pt.y - kasi->alku.y;
  if ( dx || dy ) kasi->suunta = atan2(-dy,dx);

  PiirraNotKasi(hDC,kasi);   /* Piirretn uusi oikea ksi tilalle  */

  return 1;
}

/****************************************************************************/
void MyDraw(HWND hWnd,HDC hDC)
{
#pragma argsused
/*  SelectObject(hDC,GetStockObject(BLACK_BRUSH)); */
  HBRUSH hBrRed,vanha;
  hBrRed = CreateSolidBrush(RGB(255,0,0));
  vanha = SelectObject(hDC,hBrRed);
  Ellipse(hDC,40,40,80,80);   /* P */
  SelectObject(hDC,vanha);
  DeleteObject(hBrRed);

  MoveTo(hDC, 60, 80);
  LineTo(hDC, 60,180);        /* Keskivartalo */
  LineTo(hDC, 20,260);        /* Vasen jalka  */
  MoveTo(hDC, 60,180);
  LineTo(hDC,100,260);        /* Oikea jalka  */
  MoveTo(hDC, 20,170);
  LineTo(hDC, 60, 90);        /* Vasen ksi   */
  PiirraNotKasi(hDC,&Oikea);  /* Oikea ksi   */
}


/****************************************************************************/
/*
** Seuraavissa aliohjelmissa on seuraavat parametrit:
**
**   hWnd     - ikkunan kahva
**   wParam   - SHIFT, CTRL ja ALT -nppimen tile, voidaan testata esim.
**                 if ( wParam && MK_CONTROL ) ... hommat jos CTRL alhaalla
**   lParam   - x = LOWORD(lParam) ja y = HIWORD(lParam)
**              ktevmpi on kuitenkin selvitt hiren paikka:
**               POINT pt = MAKEPOINT(lParam)
**
----------------------------------------------------------------------------*/

/****************************************************************************/
void MyDown(HWND hWnd, WORD wParam, LONG lParam)
/* Aliohjelmaa kutsutaan kun hiiren nappa on painettu alas                  */
{
#pragma argsused
}


/****************************************************************************/
void MyUp  (HWND hWnd, WORD wParam, LONG lParam)
/* Aliohjelmaa kutsutaan kun hiiren nappi on pstetty yls                 */
{
#pragma argsused
}


/****************************************************************************/
void MyMove(HWND hWnd, WORD wParam, LONG lParam)
/* Aliohjelmaa kutsutaan kun hiiri on liikkunut                             */
{
#pragma argsused
  HDC     hDC;

  if ( ( hDC = GetDC(hWnd) ) == NULL ) return;

  muuta_kaden_suunta(hDC,lParam,&Oikea);

  ReleaseDC(hWnd,hDC);
}