/**************/
/* timeman.c  */
/****************************************************************************/
/*
** Malliohjelma "animaatiosta":
** Tikku-ukko osoittaa oikealla kdelln miss hiiri menee.
** Vasen ksi pyrii ajan mukana. Aika nkyy mys ikkunan otsikossa.
** Aika voidaan nollata painamalla hiiren nappulaa.
**
** Kytt: Tee projekti show_man.prj jossa on:
**           samplewt.c
**           time_man.c
**           samplewt.def
**         ja aja.
**
** Vesa Lappalainen 15.8.1992
**
** Muuten tietoa animaatiosta ks. Show_man.c
**
** Tehtvi: 1) Onko kello tarkka?
**
**           2) Miten kello saataisiin tarkaksi?
**
**           3) Kirjoita ohjelma, miss pallo pomppii abs(sin(x)):n muotoista
**              rataa edestakaisin vlill 0-400 (eli kun tulee
**              koordinaattiin 400, kntyy takaisin ja kun
**              tulee koordinaattiin 0, kntyy  taas takaisin)
**
**           4) Muuta pomppimisohjelmaa siten, ett kun pallo trm ikkunan
**              reunaan, muutetaan suuntaa.  Ikkunan koko saadaan selville
**              kutsulla GetClientRect (ks. Help).
**
**           5) Kirjoita "Tennis"-peli, miss pallo kulkee suoraviivaista
**              rataa ja aina trmtessn ikkunan seinn, kimpoaa oikeaan
**              suuntaan.  Mys trmtessn kursoriin, muuttaa
**              pallo suuntaansa.
**
----------------------------------------------------------------------------*/
#include <windows.h>
#include <math.h>
#include <stdio.h>

char *WindowName = "Aika-ukko";

typedef struct {
  int    pituus;
  double suunta;
  POINT  alku;
} kasi_tyyppi;


static kasi_tyyppi Oikea = { 80, 0.0, {60,90} };
static kasi_tyyppi Vasen = { 80, M_PI, {60,90} };

#define AIKA_VALI 50  /* ms */
static double aika = 0;

/****************************************************************************/
#define POINT_TO_2(pt) pt.x,pt.y  /* Pisteest 2 erillist lukua            */

#define POINT_PLUS_RA(pt,r,a) \
            pt.x+r*cos(a),pt.y-r*sin(a) /* Piste + r*suunta                 */


/****************************************************************************/
void PiirraNotKasi(HDC hdc,kasi_tyyppi *kasi)
/* Piirretn ukon oikea ksi NOT-kynll.                                  */
{
  SelectObject(hdc,GetStockObject(NULL_BRUSH));
  SetROP2(hdc,R2_NOT);
  MoveTo(hdc,POINT_TO_2(kasi->alku));
  LineTo(hdc,POINT_PLUS_RA(kasi->alku,kasi->pituus,kasi->suunta));

}


/****************************************************************************/
int muuta_kaden_suunta(HDC hDC, LONG lParam, kasi_tyyppi *kasi)
{
  int dx,dy;
  POINT pt = MAKEPOINT(lParam);

  PiirraNotKasi(hDC,kasi);   /* Pyyhitn vanha ksi  pois    */

  dx = pt.x - kasi->alku.x;
  dy = pt.y - kasi->alku.y;
  if ( dx || dy ) kasi->suunta = atan2(-dy,dx);

  PiirraNotKasi(hDC,kasi);   /* Piirretn uusi oikea ksi tilalle  */

  return 1;
}

/****************************************************************************/
void MyDraw(HWND hWnd, HDC hDC)
{
#pragma argsused
  Ellipse(hDC,40,40,80,80);   /* P */
  MoveTo(hDC, 60, 80);
  LineTo(hDC, 60,180);        /* Keskivartalo */
  LineTo(hDC, 20,260);        /* Vasen jalka  */
  MoveTo(hDC, 60,180);
  LineTo(hDC,100,260);        /* Oikea jalka  */
  MoveTo(hDC, 20,170);
  PiirraNotKasi(hDC,&Vasen);  /* Oikea ksi   */
  PiirraNotKasi(hDC,&Oikea);  /* Oikea ksi   */
}

/****************************************************************************/
void InitMy(HWND hWnd)
/* Ajastin tytyy alustaa jotta se toimisi.                                 */
{
  SetTimer(hWnd,1,AIKA_VALI,NULL);
}


/****************************************************************************/
void MyTimer(HWND hWnd, WORD wParam, LONG lParam)
/* Aliohjelmaa kutsutaan kun valittu aika on tullut tyteen                 */
{
#pragma argsused
  HDC     hDC;
  char    st[80];
  static  laskuri = 0;

  if ( ( hDC = GetDC(hWnd) ) == NULL ) return;

  PiirraNotKasi(hDC,&Vasen);   /* Pyyhitn vanha ksi  pois    */

  if ( ( Vasen.suunta -= M_PI/45.0 ) < 0 ) Vasen.suunta = 2*M_PI;

  PiirraNotKasi(hDC,&Vasen);   /* Piirretn uusi oikea ksi tilalle  */


  aika += AIKA_VALI;

  if ( ++laskuri >= 1000/AIKA_VALI ) {
    sprintf(st,"Aika-ukko: %8.2lf",aika/1000.0);
    SetWindowText(hWnd,st);
    laskuri = 0;
  }

  ReleaseDC(hWnd,hDC);
}


/****************************************************************************/
/*
** Seuraavissa aliohjelmissa on seuraavat parametrit:
**
**   hWnd     - ikkunan kahva
**   wParam   - SHIFT, CTRL ja ALT -nppimen tile, voidaan testata esim.
**                 if ( wParam && MK_CONTROL ) ... hommat jos CTRL alhaalla
**   lParam   - x = LOWORD(lParam) ja y = HIWORD(lParam)
**              ktevmpi on kuitenkin selvitt hiren paikka:
**               POINT pt = MAKEPOINT(lParam) 
**
----------------------------------------------------------------------------*/

/****************************************************************************/
void MyDown(HWND hWnd, WORD wParam, LONG lParam)
/* Aliohjelmaa kutsutaan kun hiiren nappa on painettu alas                  */
{
#pragma argsused
  aika = 0;
}


/****************************************************************************/
void MyUp  (HWND hWnd, WORD wParam, LONG lParam)
/* Aliohjelmaa kutsutaan kun hiiren nappi on pstetty yls                 */
{
#pragma argsused
  HDC     hDC;
  if ( ( hDC = GetDC(hWnd) ) == NULL ) return;
  PiirraNotKasi(hDC,&Vasen);   /* Pyyhitn vanha ksi  pois    */
  Vasen.suunta = M_PI;
  PiirraNotKasi(hDC,&Vasen);   /* Piirretn uusi oikea ksi tilalle  */
  ReleaseDC(hWnd,hDC);
}


/****************************************************************************/
void MyMove(HWND hWnd, WORD wParam, LONG lParam)
/* Aliohjelmaa kutsutaan kun hiiri on liikkunut                             */
{
#pragma argsused
  HDC     hDC;

  if ( ( hDC = GetDC(hWnd) ) == NULL ) return;

  muuta_kaden_suunta(hDC,lParam,&Oikea);

  ReleaseDC(hWnd,hDC);
}