/**************/
/* kolmio.c   */
/****************************************************************************/
/*
** Malliohjelma kauan kestvst piirtmisest.
** Piirretn rekursiivisesti joukko kolmioita.
**
** Vesa Lappalainen 16.8.1992 & 14.6.1994
**
** Kntminen:
**  Tee projektio kolmio.prj:
**     kolmio.c
**     simplet.def
**     ALI\timer.c
**     ALI\tabhand.c
**
** Tehtvi 1)  Kokeile piirtmisen aikana valita ikkunan suurennus tms. tai
**              paina vaikkapa kahdesti (molemmat vlill irti) ALT-TAB.
**              Sd ohjelman suoritusaikaa muuttelemalla vakiota
**              PIENIN_KOLMIO (pieni->kauan, iso-> nopea).
**              Mik vika on ohjelmassa?
**
**              Kynnist samanaikaisesti jokin toinen ohjelma ja sd sen
**              ikkunan niin pieneksi, ettei se peit kuin osaksi "Kolmiot"
**              ikkunaa.  Laita sitten ikkunoiden jrjestys siten, ett
**              "Kolmiot"-ikkuna on toiseksi ylin ja avaamasi pikku-ikkuna
**              on ylimpn.  Siirtele pikkuikkunaa kolmiot ikkunan pll!
**
**          2)  Ohjelmaikkunan otsikkoksi muuttuu aika, jonka ikkunan uudelleen
**              piirtminen vei.  Kokeile peitt ikkunaa eri tavoin toisella
**              ikkunalla ja koeta keksi syy, miksi uudelleen piirto vie
**              eri ajan peiton suuruudesta riippuen.
**
**          3)  Ohjelmaan liittyy mys "hp-hp"-piirtminen.  Miten luulet
**              piirtmisen toimivan?  Ky vlill toisessa ikkunassa ja
**              palaa takaisin ohjelmaan.  Mit ky piirroksille?
**
**          4)  Muuta "piirtminen" sellaiseksi, ett kun hiiren nappi
**              painetaan alas, aletaan piirtmn viivaa kunnes
**              nappi pstetn yls.  Mit tapahtuu kun nappi
**              pstetnkin yls "Kolmiot"-ikkunan ulkopuolella ja
**              sitten tuodaan hiiri takaisin ikkunan sislle nappi
**              ylhll?
**
**          5)  Miten "Piirtminen" pitisi muuttaa, jotta kuva korjautuisi
**              toisessa ikkunassa kymisen jlkeen?
**
----------------------------------------------------------------------------*/
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "timer.h"
#include "tabhand.h"

/***************************************************************************/
TblClassSWindowMAIN("TriangleClass",0,"Kolmio",MsgTbl,0);
/***************************************************************************/

#define PIENIN_KOLMIO 0.2   /* Sd tll kauanko koko kuvan piirt. kest.  */

#define I(x,y) hdc,((int)(x)),((int)(y))

/****************************************************************************/
void kolmio(HDC hdc, double x, double y, double h)
{
  double s2 = h / (sqrt(3));

  MoveToEx(I(x,y),NULL);
  LineTo(I(x-s2,y-h));
  LineTo(I(x+s2,y-h));
  LineTo(I(x,y));

  if ( h < PIENIN_KOLMIO ) return;

  kolmio(hdc,x-s2,y,h/2);  /* Pienempi kolmio vasemmalle */
  kolmio(hdc,x+s2,y,h/2);  /* Pienempi kolmio oikealle   */
  kolmio(hdc,x,y-h,h/2);   /* Pienempi kolmio ylpuolelle*/
}


/****************************************************************************/
static EVENT WM_paint(tMSGParam *msg) /* # MAKE_DC # */
{
  char s[100];

  SetWindowText(msg->hWnd,"Kolmio: Piirretn...");

  start_timer(1);

  kolmio(msg->hDC,300,400,200);

  sprintf(s,"Kolmio: %5.2lf s.",stop_timer(1));

  SetWindowText(msg->hWnd,s);

  return NULL;
}


/****************************************************************************/
/*
** Seuraavissa aliohjelmissa on seuraavat parametrit:
**
**   hWnd     - ikkunan kahva
**   wParam   - SHIFT, CTRL ja ALT -nppimen tile, voidaan testata esim.
**                 if ( wParam && MK_CONTROL ) ... hommat jos CTRL alhaalla
**   lParam   - x = LOWORD(lParam) ja y = HIWORD(lParam)
**              ktevmpi on kuitenkin selvitt hiren paikka:
**               POINTS pt = MAKEPOINTS(lParam)
**
----------------------------------------------------------------------------*/

static POINTS old = {0,0};
static pen_down  = 0;

/****************************************************************************/
static EVENT WM_lbuttondown(tMSGParam *msg)
/* Aliohjelmaa kutsutaan kun hiiren nappa on painettu alas                  */
{
  old = MAKEPOINTS(msg->lParam);
  pen_down = 1;
  return 0;
}

/****************************************************************************/
static EVENT WM_lbuttonup(tMSGParam *msg)
/* Aliohjelmaa kutsutaan kun hiiren nappi on pstetty yls                 */
{
#pragma argsused
  pen_down = 0;
  return 0;
}

/****************************************************************************/
static EVENT WM_mousemove(tMSGParam *msg) /* # MAKE_DC # */
/* Aliohjelmaa kutsutaan kun hiiri on liikkunut                             */
{
  POINTS pt = MAKEPOINTS(msg->lParam);

  if ( !pen_down ) return 0;

  MoveToEx(msg->hDC,old.x,old.y,NULL);
  LineTo(msg->hDC,pt.x,pt.y);
  return 0;
}

/****************************************************************************/
static EVENT WM_create(tMSGParam *msg)
/* Aliohjelmaa kutsutaan kun ikkuna on luotu, muttei viel nytss.        */
{
  MoveWindow(msg->hWnd,10,10,600,500,FALSE);
  return 0;
}

/****************************************************************************/
/* Viestien ksittelytaulukko                                               */
/****************************************************************************/
tMSGEntry MsgTbl[] = {
  EV_HANDLE_WM_DESTROY,
  { WM_PAINT , DoC , DoC , WM_paint,  MAKE_DC  }, /*a*/
  { WM_LBUTTONDOWN , DoC , DoC , WM_lbuttondown }, /*a*/
  { WM_LBUTTONUP , DoC , DoC , WM_lbuttonup }, /*a*/
  { WM_MOUSEMOVE , DoC , DoC , WM_mousemove,  MAKE_DC  }, /*a*/
  { WM_CREATE , DoC , DoC , WM_create }, /*a*/
  { 0 }
};
/****************************************************************************/
