/**************/
/* show_man.c */
/****************************************************************************/
/*
** Malliohjelma "animaatiosta":
** Tikku-ukko osoittaa oikealla kdelln miss hiiri menee.
**
** Kytt: Tee projekti show_man jossa on:
**           show_man.c
**           simplet.def
**           ALI\tabhand.c
**           ja aja.
**
** Vesa Lappalainen 15.8.1992 & 3.9.1994
**
** Animaation idea:
**   1. kuva on piirretty valmiiksi
**   2. kun jotakin osaa kuvasta pit siirt, pyyhitn vanha osa pois
**   3. lasketaan osan uusi paikka ja/tai muoto
**   4. piirretn uudistettu osa
**
**   "Rautalankamallissa" pyyhkiminen ky siten, ett piirretn
**   kuvan osa kynll, joka muuttaa kaikki pisteet pinvastaisen
**   vrisiksi (mm. musta piste muuttuu valkoiseksi ja pinvastoin).
**   Kun tllaisella kynll piirretn kaksi kertaa sama kuva, ei
**   tuloksena olekaan mitn, siis joka toisella kerralla kuva hvi.
**
**   Monimutkaisemmassa tapauksessa kuvan osan alla oleva taustakuva
**   pit ensin tallettaa ja sitten piirt vasta kuvan osa.
**   Siirrossa tausta palautetaan ja luetaan uusi tausta uudesta
**   paikasta ja sitten piirretn uusi osa.
**
** Animaatio:
** ==========
**           1)  Oikea ja vasen ksi eivt ole yhtpitki!
**               Korjaa ohjelmaa siten, ett ne ovat yhtpitki.
**
**           2)  Muuta ohjelmaa siten, ett kun hiiren nappi painetaan
**               alas, kntyy vasen ksi osoittamaan hiiren suuntaa
**               painamishetkell.  Oikea ksi osoittaa koko ajan.
**
**           3)  Tee sama vasemmalle jalalle.  Miten muuttujien ja tyyppien
**               nimet tllin kannattaa muuttaa?
**
**           4)  Piirr ukolle punainen p.
**
**           5)  Muuta ohjelmaa siten, ett aluksi oikea ksi osoittaa kursoria
**               Kun nappi painastaan alas, lukittuu oikea ksi paikalleen ja
**               oikea jalka rupeaa osoittamaan hiirt niin kauan kun nappi
**               on alhaalla.  Kun nappi pstet yls, alkaa vasen ksi
**               osoittamaan kursoria ja kun nappi painetaan alas alkaa
**               vasen jalka ja kun nappi pstetn yls, ollaan jlleen
**               alkuperisess tilanteessa, eli oikea ksi osoittaa jne...
----------------------------------------------------------------------------*/
#include <windows.h>
#include <math.h>
#include <stdio.h>
#include "tabhand.h"

/***************************************************************************/
TblClassSWindowMAIN("ShowmanClass",0,"Nyttj-ukko",MsgTbl,0);
/***************************************************************************/

typedef struct {
  int    pituus;
  double suunta;
  POINT  alku;
} kasi_tyyppi;

#define RAD(d) ((d)*M_PI/180.0)

static kasi_tyyppi Oikea = { 80, 0.0, {60,90} };

/****************************************************************************/
#define POINT_TO_2(pt) pt.x,pt.y  /* Pisteest 2 erillist lukua            */

/* point + radius,angle (radians)         */
#define POINT_PLUS_RA(pt,r,a) \
            pt.x+r*cos(a)+0.5,pt.y-r*sin(a)+0.5 /* Piste + r*suunta         */


/****************************************************************************/
static void PiirraKasi(HDC hdc,kasi_tyyppi *kasi)
/* Piirretn ukon kasi.                                                    */
{
  MoveToEx(hdc,POINT_TO_2(kasi->alku),NULL);
  LineTo(hdc,POINT_PLUS_RA(kasi->alku,kasi->pituus,kasi->suunta));
}

/****************************************************************************/
static void PiirraNotKasi(HDC hdc,kasi_tyyppi *kasi)
/* Piirretn ukon kasi NOT-kynll.                                        */
{
  int oldROP = SetROP2(hdc,R2_NOT);
  PiirraKasi(hdc,kasi);
  SetROP2(hdc,oldROP);
}

/****************************************************************************/
static int muuta_kaden_suunta(HDC hDC, LONG lParam, kasi_tyyppi *kasi)
{
  int dx,dy;
  POINTS pt = MAKEPOINTS(lParam);

  PiirraNotKasi(hDC,kasi);   /* Pyyhitn vanha kasi pois    */

  dx = pt.x - kasi->alku.x;
  dy = pt.y - kasi->alku.y;
  if ( dx || dy ) kasi->suunta = atan2(-dy,dx); else kasi->suunta=0;

  PiirraNotKasi(hDC,kasi);   /* Piirretn uusi kasi tilalle  */

  return 1;
}

/****************************************************************************/
static EVENT WM_paint(tMSGParam *msg) /* # MAKE_DC # */
{
  Ellipse(msg->hDC,40,40,80,80);     /* P */
  MoveToEx(msg->hDC, 60, 80,NULL);
  LineTo(  msg->hDC, 60,180);        /* Keskivartalo */
  LineTo(  msg->hDC, 20,260);        /* Vasen jalka  */
  MoveToEx(msg->hDC, 60,180,NULL);
  LineTo(  msg->hDC,100,260);        /* Oikea jalka  */
  MoveToEx(msg->hDC, 20,170,NULL);
  LineTo(  msg->hDC, 60, 90);        /* Vasen ksi   */
  PiirraNotKasi(msg->hDC,&Oikea);    /* Oikea ksi   */
  return 0;
}

/****************************************************************************/
static EVENT WM_mousemove(tMSGParam *msg) /* # MAKE_DC # */
/* Aliohjelmaa kutsutaan kun hiiri on liikkunut                             */
{
  muuta_kaden_suunta(msg->hDC,msg->lParam,&Oikea);
  return 0;
}

/****************************************************************************/
static EVENT WM_create(tMSGParam *msg)
/* Aliohjelmaa kutsutaan kun ikkuna on luotu, muttei viel nytss.        */
{
  MoveWindow(msg->hWnd,10,10,600,500,FALSE);
  return 0;
}

/****************************************************************************/
/* Viestien ksittelytaulukko                                               */
/****************************************************************************/
tMSGEntry MsgTbl[] = {
  EV_HANDLE_WM_DESTROY,
  { WM_PAINT , DoC , DoC , WM_paint,  MAKE_DC  }, /*a*/
  { WM_MOUSEMOVE , DoC , DoC , WM_mousemove,  MAKE_DC  }, /*a*/
  { WM_CREATE , DoC , DoC , WM_create }, /*a*/
  { 0 }
};
/****************************************************************************/
