/**************/
/* timeman.c  */
/****************************************************************************/
/*
** Malliohjelma "animaatiosta":
** Tikku-ukko osoittaa oikealla kdelln miss hiiri menee.
** Vasen ksi pyrii ajan mukana. Aika nkyy mys ikkunan otsikossa.
** Aika voidaan nollata painamalla hiiren vasenta nappulaa.
**
** Kytt: Tee projekti time_man.prj jossa on:
**           time_man.c
**           simplet.def
**           ALI\timer.c
**           ALI\tabhand.c
**           ja aja.
**
** Vesa Lappalainen 15.8.1992 & 13.6.1994
**
** Animaation idea:
**   1. kuva on piirretty valmiiksi
**   2. kun jotakin osaa kuvasta pit siirt, pyyhitn vanha osa pois
**   3. lasketaan osan uusi paikka ja/tai muoto
**   4. piirretn uudistettu osa
**
**   "Rautalankamallissa" pyyhkiminen ky siten, ett piirretn
**   kuvan osa kynll, joka muuttaa kaikki pisteet pinvastaisen
**   vrisiksi (mm. musta piste muuttuu valkoiseksi ja pinvastoin).
**   Kun tllaisella kynll piirretn kaksi kertaa sama kuva, ei
**   tuloksena olekaan mitn, siis joka toisella kerralla kuva hvi.
**
**   Monimutkaisemmassa tapauksessa kuvan osan alla oleva taustakuva
**   pit ensin tallettaa ja sitten piirt vasta kuvan osa.
**   Siirrossa tausta palautetaan ja luetaan uusi tausta uudesta
**   paikasta ja sitten piirretn uusi osa.
**
** Animaatio:
** ==========
** Tehtvi: 1)  Ukolla on seuraava vika:
**               - kynnist ukko-ohjelma
**               - aktivoi jokin pieni ikkuna, esim. WinCD
**               - siirr ko. pieni ikkuna ukon vartalon oikealle puolelle
**               - odota kunnes kellon mukana liikkuva ksi kiertyy
**                 tmn pikkuikkunan alle
**               - aktivoi ukkoikkuna ennenkuin ksi tulee pikkuikkunan
**                 takaa nkyviin
**               Mist vika johtuu?  Miten vika voitaisiin korjata?
**               (Vian syyt voit tutkia asettamalla vakion TUTKI_VIESTIT
**                1:ksi, jolloin net viestien saapumisjrjestyksen.)
**
**           2)  Muuta ohjelmaa siten, ett kun hiiren nappi painetaan
**               alas, kntyy oikea jalka osoittamaan hiiren suuntaa
**               painamishetkell.  Oikea ksi osoittaa koko ajan.
**
**           3)  Muuta ohjelmaa siten, ett aluksi oikea ksi osoittaa kursoria
**               Kun oikea nappi painastaan alas, lukittuu oikea ksi paikalleen
**               ja vasen jalka rupeaa osoittamaan hiirt niin kauan kun nappi
**               on alhaalla.  Kun nappi pstet yls, alkaa oikea jalka
**               osoittamaan kursoria ja kun nappi painetaan alas alkaa
**               oikea ksi osoittamaan jne...
** Kello:
** ======
** Tehtvi: 1) Onko kello tarkka?
**
**           2) Miten kello saataisiin tarkaksi?
**
**           3) Kirjoita ohjelma, miss pallo pomppii abs(sin(x)):n muotoista
**              rataa edestakaisin vlill 0-400 (eli kun tulee
**              koordinaattiin 400, kntyy takaisin ja kun
**              tulee koordinaattiin 0, kntyy  taas takaisin)
**
**           4) Muuta pomppimisohjelmaa siten, ett kun pallo trm ikkunan
**              reunaan, muutetaan suuntaa.  Ikkunan koko saadaan selville
**              kutsulla GetClientRect (ks. Help).
**
**           5) Kirjoita "Tennis"-peli, miss pallo kulkee suoraviivaista
**              rataa ja aina trmtessn ikkunan seinn, kimpoaa oikeaan
**              suuntaan.  Mys trmtessn kursoriin, muuttaa
**              pallo suuntaansa.
**
----------------------------------------------------------------------------*/
#include <windows.h>
#include <math.h>
#include <stdio.h>
#include "tabhand.h"
#define TUTKI_VIESTIT 1 /* Halutaanko tutkia viestien tulojrjestyst */

/***************************************************************************/
TblClassSWindowMAIN("TimemanClass",0,"Aika-ukko",MsgTbl,0);
/***************************************************************************/

typedef struct {
  int    pituus;
  double suunta;
  POINT  alku;
} raaja_tyyppi;

#define RAD(d) ((d)*M_PI/180.0)

static raaja_tyyppi Raajat[4] = {
/* pituus,suunta    , alku.x alku,y                   */
  { 80, RAD(180)    , { 60, 90} }, /* 0 = Vasen ksi  */
  { 80, RAD(30)     , { 60, 90} }, /* 1 = Oikea ksi  */
  { 90, RAD(180+60) , { 60,180} }, /* 2 = Vasen jalka */
  { 90, RAD(-60)    , { 60,180} }  /* 3 = Oikea jalka */
};

static raaja_tyyppi *Vasen_kasi = &Raajat[0];
static raaja_tyyppi *Oikea_kasi = &Raajat[1];

#define AIKA_VALI 50  /* ms */
#define TARKKUUS 1000 /* kellon nytt nin usein */

static double aika = 0; /* ms */
static double vasen_alhaalla = 0; /* Aika ei etene jos vasen on alhaalla!   */

/****************************************************************************/
#define POINT_TO_2(pt) pt.x,pt.y  /* Pisteest 2 erillist lukua            */

#define POINT_PLUS_RA(pt,r,a) \
            pt.x+r*cos(a)+0.5,pt.y-r*sin(a)+0.5 /* Piste + r*suunta         */


/****************************************************************************/
static void PiirraRaaja(HDC hdc,raaja_tyyppi *raaja)
/* Piirretn ukon raaja.                                                   */
{
  MoveToEx(hdc,POINT_TO_2(raaja->alku),NULL);
  LineTo(hdc,POINT_PLUS_RA(raaja->alku,raaja->pituus,raaja->suunta));
}

/****************************************************************************/
static void PiirraNotRaaja(HDC hdc,raaja_tyyppi *raaja)
/* Piirretn ukon raaja NOT-kynll.                                       */
{
  int oldROP = SetROP2(hdc,R2_NOT);
  PiirraRaaja(hdc,raaja);
  SetROP2(hdc,oldROP);
}

/****************************************************************************/
static int muuta_raajan_suunta(HDC hDC, LONG lParam, raaja_tyyppi *raaja)
{
  int dx,dy;
  POINTS pt = MAKEPOINTS(lParam);

  PiirraNotRaaja(hDC,raaja);   /* Pyyhitn vanha raaja pois    */

  dx = pt.x - raaja->alku.x;
  dy = pt.y - raaja->alku.y;
  if ( dx || dy ) raaja->suunta = atan2(-dy,dx); else raaja->suunta=0;

  PiirraNotRaaja(hDC,raaja);   /* Piirretn uusi raaja tilalle  */

  return 1;
}

/****************************************************************************/
static int muuta_raajan_suunta_rad(HDC hDC, double rad, raaja_tyyppi *raaja)
{
  PiirraNotRaaja(hDC,raaja);   /* Pyyhitn vanha raaja pois    */

  raaja->suunta = fmod(rad,2*M_PI);

  PiirraNotRaaja(hDC,raaja);   /* Piirretn uusi raaja tilalle  */

  return 1;
}


/****************************************************************************/
#if TUTKI_VIESTIT /* Tl voidaan tutkia viestien tulojrjestyst          */
# include <string.h>
# define T "--"
# define MV 10    /* Nytettvien viestien maksimimr.                    */
  static char *v[MV];
  static int paikka = 0;
  /* V   = laita ja nyt Viesti n
  ** LV  = Lis Viestit.  Jonossa st tytyy jo olla jotakin (strcat => vaara!)
  ** TV  = Tyhjenn Viestien nytt
  */
# define V(n) {v[paikka]=n; paikka++; if (paikka>=MV) paikka=MV-1; else nayta_aika(msg); }
# define LV() {int i;for (i=0; i<MV; i++) {strcat(st," "); strcat(st,v[i]?v[i]:T);}}
# define TV() {int i;for (i=0; i<MV; i++) v[i]=T; paikka=0; nayta_aika(msg); }
#else /* ei TUTKI_VIESTIT */
# define V(n)
# define LV()
# define TV()
#endif /* TUTKI_VIESTIT */
/****************************************************************************/

/****************************************************************************/
static void nayta_aika(tMSGParam *msg)
/* Nytetn kellonaika sekunteina ikkunan otsikkopalkissa                  */
{
  char st[200]; 
  sprintf(st,"Aika-ukko: %8.2lf ",aika/1000.0);
  LV();
  SetWindowText(msg->hWnd,st);
}


/****************************************************************************/
static EVENT WM_paint(tMSGParam *msg) /* # MAKE_DC # */
{
  int i;
  Ellipse(msg->hDC,40,40,80,80);   /* P */
  MoveToEx(msg->hDC, 60, 80, NULL);
  LineTo(msg->hDC, 60,180);        /* Keskivartalo */
  for (i=0; i<4; i++)
    PiirraNotRaaja(msg->hDC,&Raajat[i]);  /* Jsenet      */
  V("PA");
  nayta_aika(msg);
  return 0;
}


/****************************************************************************/
static void InitMy(tMSGParam *msg)
/* Ajastin tytyy alustaa jotta se toimisi.                                 */
{
  SetTimer(msg->hWnd,1,AIKA_VALI,NULL);
}

/****************************************************************************/
static EVENT WM_timer_1(tMSGParam *msg) /* # MAKE_DC # */ 
/* Aliohjelmaa kutsutaan kun valittu aika on tullut tyteen                 */
{
  static int laskuri = 0; /* Aikaa ei kannata nytt kokoajan, vain sek vl*/

  if ( vasen_alhaalla ) return 0;

  if ( aika == 0 ) laskuri = 0;

  muuta_raajan_suunta_rad(msg->hDC,Vasen_kasi->suunta-M_PI/45.0,Vasen_kasi);

  aika += AIKA_VALI;

  if ( ++laskuri >= TARKKUUS/AIKA_VALI ) {
    laskuri = 0;
    nayta_aika(msg);
  }

  return 0;
}


/****************************************************************************/
/*
** Seuraavissa aliohjelmissa on seuraavat parametrit:
**
**   hWnd     - ikkunan kahva
**   wParam   - SHIFT, CTRL ja ALT -nppimen tile, voidaan testata esim.
**                 if ( wParam && MK_CONTROL ) ... hommat jos CTRL alhaalla
**   lParam   - x = LOWORD(lParam) ja y = HIWORD(lParam)
**              ktevmpi on kuitenkin selvitt hiren paikka:
**               POINT pt = MAKEPOINT(lParam)
**
----------------------------------------------------------------------------*/

/****************************************************************************/
static EVENT WM_lbuttondown(tMSGParam *msg)  /* # MAKE_DC # */
/* Aliohjelmaa kutsutaan kun hiiren nappi on painettu alas                  */
{
  aika = 0;
  vasen_alhaalla = 1;
  muuta_raajan_suunta_rad(msg->hDC,M_PI,Vasen_kasi);
  V("LD");
  nayta_aika(msg);
  return 0;
}


/****************************************************************************/
static EVENT WM_lbuttonup(tMSGParam *msg)
/* Aliohjelmaa kutsutaan kun hiiren nappi on pstetty yls                 */
{
#pragma argsused
  V("LU");
  vasen_alhaalla = 0;
  return 0;
}

/****************************************************************************/
static EVENT WM_mousemove(tMSGParam *msg) /* # MAKE_DC # */
/* Aliohjelmaa kutsutaan kun hiiri on liikkunut                             */
{
  muuta_raajan_suunta(msg->hDC,msg->lParam,Oikea_kasi);
  return 0;
}

/****************************************************************************/
static EVENT WM_create(tMSGParam *msg)
/* Aliohjelmaa kutsutaan kun ikkuna on luotu, muttei viel nytss.        */
{
  InitMy(msg);
  nayta_aika(msg);
  V("CR");
  return 0;
}

/****************************************************************************/
static EVENT WM_rbuttondown(tMSGParam *msg)
{
#pragma argsused
  TV();
  return 0;
}

/****************************************************************************/
/* Viestien ksittelytaulukko                                               */
/****************************************************************************/
tMSGEntry MsgTbl[] = {
  EV_HANDLE_WM_DESTROY,
  { WM_PAINT , DoC , DoC , WM_paint,  MAKE_DC  }, /*a*/
  { WM_TIMER , 1 , DoC , WM_timer_1,  MAKE_DC  }, /*a*/
  { WM_LBUTTONDOWN , DoC , DoC , WM_lbuttondown,  MAKE_DC  }, /*a*/
  { WM_LBUTTONUP , DoC , DoC , WM_lbuttonup }, /*a*/
  { WM_MOUSEMOVE , DoC , DoC , WM_mousemove,  MAKE_DC  }, /*a*/
  { WM_CREATE , DoC , DoC , WM_create }, /*a*/
  { WM_RBUTTONDOWN , DoC , DoC , WM_rbuttondown }, /*a*/
  { 0 }
};
/****************************************************************************/
