/****************************************************************************

    PROGRAM: whello.c
    Project: whello.c, whello.def
    EDITOR:  Vesa Lappalainen malliohjelmien mukaisesti
    DATE:    10.8.1992

    PURPOSE: "Pienin Windows-ohjelma".  Tulostaa nyttn tekstin
             Hello World tai sen tekstin, mik on annettu ohjelman
             kutsun yhteydess (esim.  whello Terve!).

             Lisksi tulostetaan esimerkin vuoksi esiintymn kahva
             (kokonaisluku) ja mahdollisen edellisen esiintymn kahva.

    FUNCTIONS:

        WinMain()         - kutsuu alustuksia ja pyritt viestisilmukkaa
                            (tt nime ei saa vaihtaa, muiden funktioiden
                            nimet saa vaihtaa, kunhan vaihtaa kaikkialta).
        InitApplication() - alustaa ikkunaluokan
        InitInstance()    - luo pikkunan
        MainWndProc()     - ksittelee viestit

-----------------------------------------------------------------------------
  Ohjelman periaate:

  Kyttj:
    |
    |                               WHELLO:
    |     |-------------|           =======
    |     |            -+----------> WinMain
    |     |   Windows   |                InitApplication (kerrotaan MainWndProc)
    |     |   systeemi  |                InitInstance
    |-----+--->        <+------------- GetMessage (W)  <-------| viesti-
          |             |------------->TranslateMessage (W)    | silmukka
          |       -----<+------------- DispatchMessage (W)     |
          |       |     |----------------------o----------------
          |       |     |
          |       |     | viesti
          |       |--->-+----------> MainWndProc
          |            <+------------- nyttn piirtminen tai muu toimenpide
          |             |              ja viestin kuittaus (W)
          |             |
          |-------------|

-----------------------------------------------------------------------------
    COMMENTS:

        Windowsissa voi sama ohjelma olla kynniss useita kertoja
        (esiintym, instance).

        Windows-funktiot on alunperin kirjoitettu Pascalmaisiksi, joten
        ne tytyy esitett avainsanalla Pascal.

        Muuttujien ja tyyppien nimemisperiaatteita:
          h - alkuiset           : kahvoja (Handle),  kahva on kokonaisluku,
                                   jonka perusteella systeemi osaa valita
                                   jollekin toiminnalle oikean kohteen.
          wnd                    : ikkuna (WiNDow) 
          dc                     : laiteyhteys (Device Context)
          ch                     : merkki (CHar)
          sz                     : NUL-loppuinen merkkijono (String Zero)
          p                      : osoitin (pointer)
          lp                     : pitk (far) osoitin (Long Pointer)
          fn                     : funktio (FuNction)
          st                     : merkkijono (STring, yleens sz)
          a                      : taulukko (Array)
          n                      : lukumr (couNt)
          c                      : lukumr (Count)
          b                      : tavu  (Byte)
          f                      : lippu (Flag)
          WM_                    : ikkunan viesti (Window Message)
          CM_                    : komento viesti (Command Message)
          WC_                    : ikkunan luonti (Window Create)
          WS_                    : ikkunan tyyli (Window Style)
          msg                    : viesti (MeSsaGe)
          cmd                    : komento (CoMmaD)
          def                    : oletus (DEFault)
          br                     : sivellin, tyyli (BRush)
          cls                    : luokka (CLaSs)

        Esim:

          hDC                    : kahva ikkunan laiteyhteyteen
          cbClsExtra             : luokan ylimristen tavujen lkm
          hbrBackground          : kahva taustan vrin siveltimeen
          lpszMenuName           : far-osoitin Menun nimi-merkkijonoon (C-tyyli)

****************************************************************************/
#include <windows.h>            /* Tarvitaan kaikissa Windows C-ohjelmissa */
#include <windowsx.h>           /* Apumakroja ja 16/32 bittisen yht.sov    */

#include <string.h>             /* Tm ohjelma tarvitsee nm             */   
#include <stdio.h>

#define MAINWNAME "WHelloWClass"  /* Tunniste, jolla ikkunaluokka tunnetaan*/

/* Tmn ohjelman tarvitsemia globaaleja muuttujia: -----------------------*/
LPSTR  lpCallText = "Hello World"; /*Osoitin nyttn tulost. tekstille    */

HINSTANCE hPrev;         /* Edellisen esiint.nro vain nyttn tulostamiseksi*/

/****************************************************************************

    FUNCTION: MainWndProc(HWND, unsigned, WORD, LONG)

    PURPOSE:  Ksittelee pikkunalle tulevat viestit

    MESSAGES:

        WM_PAINT      - piirr ikkuna uudelleen
        WM_DESTROY    - hvit ikkuna ja lopeta ohjelma

    COMMENTS:

        Ohjelman nyttn tulostus on tss funktiossa ja sit
        kutsutaan kun tulee viesti WM_PAINT.  Mikli nin ei tehtisi,
        vaan tulostus olisi esim. fuktiossa InitInstance, niin teksti
        kyll tulisi nyttn, muttei piirtyisi uudelleen kun ikkuna
        peitettisiin tilapisesti jollakin toisella ikkunalla!

        Monimutkaisemmassa ohjelmassa tytyy pit kirjaa siit, mit
        nyttn on tulostettu, jotta ikkuna osataan piirt uudelleen
        WM_PAINT viestin tullessa.

        Nyttn piirtmist varten tytyy luoda ikkunan nyttyhteys
        (display context), joka pit hvitt kun piirtminen lopetetaan.

****************************************************************************/

LONG CALLBACK _export MainWndProc(HWND hWnd, UINT message,
                                  WPARAM wParam, LPARAM lParam)
{
    PAINTSTRUCT ps; /* BeginPaintin palauttama struktuuri, mm laiteyhdeyden*/
                    /* kahva hdc (handle device context)                   */
    char    st[80];
    HINSTANCE hInst;   /* Esiintymien numerot vain nyttn tulostamiseksi */
    HINSTANCE hpInst;

    switch (message)
    {
        case WM_PAINT:
            /* Tulostetaan nyttn haluttu teksti ja tmn sek           */
            /* edellisen esiintymn numerot, eli kahvat.                   */

            if ( BeginPaint(hWnd,&ps) ) {
              TextOut(ps.hdc, 10, 10, lpCallText, _fstrlen(lpCallText) );

              hInst = GetWindowInstance(hWnd);
              sprintf(st,"Tm kutsu: %d (0x%08X), edellinen kutsu %d (0x%08X).",
                                   hInst,hInst,               hPrev,hPrev);
              TextOut(ps.hdc, 10, 30, st, strlen(st) );
#ifndef __WIN32__ /* Huom! 2x _ */
              hpInst = (HINSTANCE)GetClassWord(hWnd,GCW_HMODULE);
#else
              hpInst = (HINSTANCE)GetClassLong(hWnd,GCL_HMODULE);
#endif
              sprintf(st,"Ohjelma:    %d (0x%08X).",hpInst,hpInst);
              TextOut(ps.hdc, 10, 50, st, strlen(st) );
              EndPaint(hWnd,&ps);
              return NULL;
            }
            break;

        case WM_DESTROY:          /* Viesti: ikkuna hvitetn             */
            PostQuitMessage(0);
            return NULL;

        default:                  /* Antaa Windowsin ksitell muut        */
            break;
    }
    return DefWindowProc(hWnd, message, wParam, lParam);
}


/****************************************************************************

    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Luo ja nytt pikkunan

    COMMENTS:

        Tt funkiota kutsutaan jokaista ohjelman kynnistyskertaa
        (esiintym, instance) varten, jotta kynnistyskerran ikkunan
        tietous saadaan talteen.

        Tt funktiota kutsutaan jokaisen uuden esiintymiskerran aluksi, eli
        sellaisten asioiden alustamiseksi, jotka ovat omia kullekin
        esiintymiskerralle.

        Tss tapauksessa luodaan ja nytetn pikkuna.


****************************************************************************/

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
    HWND    hWnd;   /* Pikkunan kahva */

    /* Luodaan pikkuna tt esiintym varten.                           */

    hWnd = CreateWindow(
        MAINWNAME,           /* Tunnus jota RegisterClass() kytti         */
        "Windows Hello",     /* Ikkunan otsikoksi tulostuva teksti         */
        WS_OVERLAPPEDWINDOW, /* Ikkunan tyyli (tavallinen)                 */
        CW_USEDEFAULT,       /* Oletus x-paikka                            */
        CW_USEDEFAULT,       /* Oletus y-paikka                            */
        CW_USEDEFAULT,       /* Oletus leveys                              */
        CW_USEDEFAULT,       /* Oletus korkeus                             */
        NULL,                /* Pikkunalla ei ole is-ikkunaa            */
        NULL,                /* Mit menua kytetn                       */
        hInstance,           /* Tm esiintym omistaa ikkunan             */
        NULL                 /* Tss ei tarvita tt osoitinta.           */
    );

    /* Jollei ikkunalle saatu kahvaa, palautetaan eponnistuminen:         */

    if ( !hWnd ) return 1;

    /* Nytetn ikkuna ja lhetetn WM_PAINT-viesti:                     */

    ShowWindow(hWnd, nCmdShow);     /* Nytetn ikkuna                    */
    UpdateWindow(hWnd);             /* Lhetetn WM_PAINT viesti          */

    return 0;
}


/****************************************************************************

    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Alustaa ikkunan tiedot ja rekisteri ikkunaluokan

    COMMENTS:

        Tt funktiota kutsutaan vain kerran, vaikka ohjelmasta olisi
        pyrimss useampia esiintymi.  Tss funktiossa tehdn ne
        alustukset, joita saa tehd vain 1. esiintymlle.

        Nyt alustetaan ikkunan luokka tyttmll ikkunatietue ja
        kutsumalla ikkunan rekisterinti RegisterClass().
        Koska kaikki esiintymt kyttvt samaa ikkunaluokkaa, riitt
        kutsun tekeminen vain yhden kerran.

****************************************************************************/

BOOL InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wc;

    /* Tytetn ikkunan luokkatietue tiedoilla, jotka kuvaavat pikkunaa */

    wc.style = NULL;                /* Oletustyyli                         */
    wc.lpfnWndProc = MainWndProc;   /* Ikkunaluokan viestit ksit. funktio */
    wc.cbClsExtra = 0;              /* Ei muuta dataa tlle luokalle       */
    wc.cbWndExtra = 0;              /* Ei muuta dataa ikkunalla            */
    wc.hInstance = hInstance;       /* Ohjelma, joka omistaa luokan        */
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION); /* Ohjelman ikoni; oletus  */
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);   /* Ohjelman kursori; oletus*/
    wc.hbrBackground = GetStockObject(WHITE_BRUSH); /* Taustan tyyli       */
    wc.lpszMenuName  = NULL;        /* Ei menuja tss ohjelmassa          */
    wc.lpszClassName = MAINWNAME;   /* Luokan tunnisteteksti, ks.          */
                                    /* CreateWindow                        */

    /* Rekisteridn luokka ja palautetaan sen onnistuminen.              */
    return RegisterClass(&wc) == NULL;
}


/****************************************************************************

    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: Kutsuu alustuksia ja huolehtii viestisilmukasta

    COMMENTS:

        Windows lyt tmn funktion nimen perusteella (WinMain), joten
        tmn funktion nime EI SAA MUUTTAA.

        Tmn funktion tulee alustaa tarvitavat yleiset resurssit ja
        aina alustaa esiintymkohtaiset (instance) resurssit.

        Tmn jlkeen yllpidetn esiintymn viestisilmukkaa.  Silmukan
        tulee ptty kun GetMessage paluttaa arvon 0.  Tm tapahtuu, kun on
        kutsuttu funktiota PostQuitMessage.  Paluussa pit palauttaa
        PostQuitMessage()-funktion palauttama arvo (joka on valmiina viestiss).

        Mikli funktiosta pit poistua ennen viestisilmukkaa, palautetaan
        arvo 1.

****************************************************************************/

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
    MSG msg;    /* Viesti */

    /* Jos ohjelman kutsussa on ollut parametrej, niin talletetaan ne     */
    if ( *lpCmdLine ) lpCallText = lpCmdLine;

    hPrev = hPrevInstance;

    if ( !hPrevInstance )       /* Onko muita esiintymi kynnisss?       */
        if ( InitApplication(hInstance) )   /* Alustetaan yhtesiset osat:  */
            return 1;           /* Pois jollei alustus onnistu             */

    /* Alustukset jotka tulevat jokaiselle esiintymlle:                   */
    if ( InitInstance(hInstance, nCmdShow) )
        return 1;

    /* Pyritetn viestisilmukkaa kunnes ohjelmasta halutaan pois         */
    while (GetMessage(&msg,     /* Viestitietue                            */
        NULL,   /* Mink ikkunan viestej tulkitaan; ohjelman kaikkien     */
        NULL,   /* Ensimmisen tulkittavan viestin nro                     */
        NULL))  /* Viimeisen tulkittavan viestin nro                       */
    {
        TranslateMessage(&msg);     /* Tulkitaan virtuaaliset np. koodit  */
        DispatchMessage(&msg);      /* Lhetetn viesti ikkunalle         */
    }
    return msg.wParam;        /* Palautetaan PostQuitMessage-funktion arvo */
}